/* Copyright (c) 2000-2001 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

//---- globals
var _findingInterps = false;
var dialog;
var wishExecutable = null;
var tclExecutable = null;
var log = getLoggingMgr().getLogger('prefs.tcl');

// preferences used in the pref window
var _elementIDs = [
    "tclshDefaultInterpreter",
    "wishDefaultInterpreter",
];


//---- functions

function PrefTcl_OnLoad()
{
    try {
        var tag = 'chrome://jaguar/content/pref/pref-tcl.xul';
        dialog = {};
    
        if (parent.hPrefWindow.prefset.hasStringPref('tclshDefaultInterpreter') &&
            parent.hPrefWindow.prefset.getStringPref('tclshDefaultInterpreter'))
            tclExecutable = parent.hPrefWindow.prefset.getStringPref('tclshDefaultInterpreter');
        else
            tclExecutable = '';
    
        if (parent.hPrefWindow.prefset.hasStringPref('wishDefaultInterpreter') &&
            parent.hPrefWindow.prefset.getStringPref('wishDefaultInterpreter'))
            wishExecutable = parent.hPrefWindow.prefset.getStringPref('wishDefaultInterpreter');
        else
            wishExecutable = '';
    
        _findingInterps = true;
        PrefTcl_InsertFindingMessage(document.getElementById("wishDefaultInterpreter"));
        PrefTcl_InsertFindingMessage(document.getElementById("tclshDefaultInterpreter"));
        PrefTcl_PopulateTclInterps();

        var origWindow = windowManager_getMainWindow();
        var cwd = origWindow.View_GetCwd();
        parent.hPrefWindow.onpageload(tag);
        var extraPaths = document.getElementById("tclExtraPaths");
        extraPaths.setCwd(cwd)
        extraPaths.init() // must happen after onpageload
        parent.hPrefWindow.registerOKCallbackFunc(PrefTcl_OkCallback, tag);
    } catch (e) {
        log.exception(e);
    }
}


function PrefTcl_OkCallback(prefset)
{
    var ok = true;
    
    // ensure that the default tcl interpreters are valid
    var defaultTclshInterp = prefset.getStringPref("tclshDefaultInterpreter");
    var defaultWishInterp  = prefset.getStringPref("wishDefaultInterpreter");
    var koSysUtils = Components.classes["@activestate.com/koSysUtils;1"].
        getService(Components.interfaces.koISysUtils);

    if (defaultTclshInterp != "") {
        if (! koSysUtils.IsFile(defaultTclshInterp)) {
            alert("No Tcl interpreter could be found at '" +
                  defaultTclshInterp + "'. You must make another " +
                  "selection for the default Tcl interpreter.\n");
            ok = false;
            document.getElementById("tclshDefaultInterpreter").focus();
        }
    }

    if (defaultWishInterp != "") {
        if (! koSysUtils.IsFile(defaultWishInterp)) {
            alert("No Tcl Wish interpreter could be found at '" +
                  defaultWishInterp + "'. You must make another selection " +
                  "for the default Tcl Wish interpreter.\n");
            ok = false;
            document.getElementById("wishDefaultInterpreter").focus();
        }
    }

    return ok;
}

function PrefTcl_PopulateTclInterps()
{
    // Populate the (tree) list of available Tcl interpreters on the current
    // system.
    
    var tclInfoEx = Components.classes["@activestate.com/koTclInfoEx;1"]
                     .createInstance(Components.interfaces.koITclInfoEx);

    var os = Components.classes["@activestate.com/koOs;1"]
                     .createInstance(Components.interfaces.koIOs);
    
    var numFound = new Object();
    var availInterps = tclInfoEx.FindInstallationPaths(numFound);
    var licensedTclsh = Array();
    var licensedWish = Array();
    var unlicensedInstalls = Array();
    
    for (var i = 0; i < availInterps.length; ++i){
        tclInfoEx.installationPath = availInterps[i]
        if (os.path.exists(tclInfoEx.tclsh_path)) {
            licensedTclsh.push(tclInfoEx.tclsh_path)
        }
        if (os.path.exists(tclInfoEx.wish_path)) {
            licensedWish.push(tclInfoEx.wish_path)
        }
    }
    
    PrefTcl_PopulateTclshInterps(licensedTclsh);
    PrefTcl_PopulateWishInterps(licensedWish);

    _findingInterps = false;
}

function PrefTcl_InsertFindingMessage(availInterpList)
{
    // remove any existing items and add a "finding..." one
    availInterpList.removeAllItems();
    availInterpList.appendItem("Finding available Tcl interpreters...");    
}

function PrefTcl_PopulateTclshInterps(availInterps)
{
    var availInterpList = document.getElementById("tclshDefaultInterpreter");
    
    availInterpList.removeAllItems();
    availInterpList.appendItem("Find on Path",'');

    var found = false;
    // populate the tree listing them
    if (availInterps.length == 0) {
        // tell the user no interpreter was found and direct them to
        // ActiveState to get one
        document.getElementById("no-avail-tclsh-interps-message").removeAttribute("collapsed");
    } else {
        for (var i = 0; i < availInterps.length; i++) {
            availInterpList.appendItem(availInterps[i],availInterps[i]);
            if (availInterps[i] == tclExecutable) found = true;
        }
    }
    if (!found && tclExecutable)
        availInterpList.appendItem(tclExecutable,tclExecutable);
}

// Populate the (tree) list of available Tcl wish interpreters on the current
// system.
function PrefTcl_PopulateWishInterps(availInterps)
{
    var availInterpList = document.getElementById("wishDefaultInterpreter");
    
    availInterpList.removeAllItems();
    availInterpList.appendItem("Find on Path",'');

    var found = false;
    // populate the tree listing them
    if (availInterps.length == 0) {
        // tell the user no interpreter was found and direct them to
        // ActiveState to get one
        document.getElementById("no-avail-wish-interps-message").removeAttribute("collapsed");
    } else {
        for (var i = 0; i < availInterps.length; i++) {
            availInterpList.appendItem(availInterps[i],availInterps[i]);
            if (availInterps[i] == wishExecutable) found = true;
        }
    }
    if (!found && wishExecutable)
        availInterpList.appendItem(wishExecutable,wishExecutable);
}

function loadTclExecutable()
{
    var tclExe = filepicker_openExeFile();
    if (tclExe != null) {
        var availInterpList = document.getElementById("tclshDefaultInterpreter");
        availInterpList.selectedItem = availInterpList.appendItem(tclExe, tclExe);
    }
}

function loadWishExecutable()
{
    var wishExe = filepicker_openExeFile();
    if (wishExe != null) {
        var availInterpList = document.getElementById("wishDefaultInterpreter");
        availInterpList.selectedItem = availInterpList.appendItem(wishExe, wishExe);
    }
}

function loadTclLogpath()
{
    var tclLog = filepicker_getFolder();
    if (tclLog != null) {
        var textbox = document.getElementById("tcl_debuggerlogpath");
        textbox.value = tclLog;
    }
}
