/* Copyright (c) 2000-2001 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

const dquote = '"';
function is_whitespace(ch) {
    return ch==" " || ch=="\t";
};

function CommandLineParser() {}

/* decorate prototype to provide ``class'' methods and property accessors */
CommandLineParser.prototype = {
    parse: function (s, ret_args, ret_num) {
        this.str = s;
        this.i = 0;
        ret = new Array();
        while (1) {
            tok = this._getNextToken();
            if (tok==null)
                break;
            ret[ret.length] = tok;
        }
        ret_args.value = ret;
        ret_num.value = ret.length;
    },
    _skipWS: function() {
        while (this.i < this.str.length) {
            var ch = this.str[this.i];
            if (is_whitespace(ch)) {
                this.i++;
                continue;
            }
            break;
        }
    },
    _skipToStartOfToken: function() {
        this._skipWS();
        if (this.i == this.str.length)
            return -1;
        var ch = this.str[this.i];
        if (ch == "\\" && this.i < this.str.length && this.str[this.i+1]==dquote)
            return ++this.i; // token starts at the escaped quote.
        this.tok_start = (ch == dquote) ? ch : '';
        return this.i;
    },
    _getNextToken: function() {
        // Dunno why - but my testing at the NT shell (dont laugh ;)
        // shows trailing quotes in the string being removed completely.
        // It must be a quote followed by WS or EOS to close the token.
        var start = this._skipToStartOfToken();
        if (start == -1)
            return null;
        var tok = "";
        var ch_needed_to_close = this.tok_start;
        while (this.i < this.str.length) {
            var ch = this.str[this.i++];
            var is_more = (this.i<this.str.length);
            if (ch == "\\" && is_more && this.str[this.i]==dquote) {
                tok += dquote;
                this.i++;
            } else if (ch==dquote) {
                // see comment above.
                if (ch==ch_needed_to_close && (!is_more || is_whitespace(this.str[this.i]))) {
                    return tok;
                }
                // This may be the first quote in a token - mark as such.
                if (!ch_needed_to_close)
                    ch_needed_to_close = ch;
                // else we just ignore the quote!
            } else if (ch_needed_to_close == '' && is_whitespace(ch)) {
                break;
            } else
                tok += ch;
        }
        return tok;
    },

    QueryInterface: function (iid) {
        if (!iid.equals(Components.interfaces.asICommandLineParser) &&
            !iid.equals(Components.interfaces.nsISupports)) {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        return this;
    }
}

var myModule = {
    firstTime: true,

    /*
     * RegisterSelf is called at registration time (component installation
     * or the only-until-release startup autoregistration) and is responsible
     * for notifying the component manager of all components implemented in
     * this module.  The fileSpec, location and type parameters are mostly
     * opaque, and should be passed on to the registerComponent call
     * unmolested.
     */
    registerSelf: function (compMgr, fileSpec, location, type) {
        dump("*** Registering CommandLine JS components\n");
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(this.myCID, 
                                    "ActiveState CommandLine Parsing Component",
                                    this.myProgID, 
                                    fileSpec, 
                                    location,
                                    type);
    },

    /*
     * The GetClassObject method is responsible for producing Factory and
     * SingletonFactory objects (the latter are specialized for services).
     */
    getClassObject: function (compMgr, cid, iid) {
        if (!cid.equals(this.myCID))
            throw Components.results.NS_ERROR_NO_INTERFACE;

        if (!iid.equals(Components.interfaces.nsIFactory))
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

        return this.myFactory;
    },

    /* CID for this class */
    myCID: Components.ID("{9b6c6b15-4b4c-4bb6-90fe-93eb85597f18}"),

    /* ProgID for this class */
    myProgID: "@activestate.com/CommandLineParser;1",

    /* factory object */
    myFactory: {
        /*
         * Construct an instance of the interface specified by iid, possibly
         * aggregating it with the provided outer.  (If you don't know what
         * aggregation is all about, you don't need to.  It reduces even the
         * mightiest of XPCOM warriors to snivelling cowards.)
         */
        createInstance: function (outer, iid) {
            if (outer != null)
                throw Components.results.NS_ERROR_NO_AGGREGATION;
            if (!iid.equals(Components.interfaces.asICommandLineParser) &&
                !iid.equals(Components.interfaces.nsISupports))
                throw Components.results.NS_ERROR_INVALID_ARG;
            return (new CommandLineParser()).QueryInterface(iid);
        }
    },

    /*
     * The canUnload method signals that the component is about to be unloaded.
     * C++ components can return false to indicate that they don't wish to be
     * unloaded, but the return value from JS components' canUnload is ignored:
     * mark-and-sweep will keep everything around until it's no longer in use,
     * making unconditional ``unload'' safe.
     *
     * You still need to provide a (likely useless) canUnload method, though:
     * it's part of the nsIModule interface contract, and the JS loader _will_
     * call it.
     */
    canUnload: function(compMgr) {
        return true;
    }
};

function NSGetModule(compMgr, fileSpec) {
    return myModule;
}
